/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Moses1;
using namespace Pythia8;

#define BINS 100

//TH1 *GLOBAL_eff = new TH1D("", "", BINS, -1., 1.);
Double_t *GLOBAL_eff = (Double_t *)malloc(BINS*sizeof(Double_t));
Double_t GLOBAL_cos1thetaAverage = 1.;
Double_t GLOBAL_cos2thetaAverage = 1.;
Double_t GLOBAL_efficiencySum    = 1.;
Double_t GLOBAL_numberOfEvents   = 1.;


Double_t physicalFitFunction(Double_t *x, Double_t *par) {
	Double_t collinsSoper  = 1.+0.5*par[0] + par[1]*x[0] + (1.-1.5*par[0])*x[0]*x[0];
	Double_t normalization = 8./3.;
	Double_t fullDistribution = collinsSoper/normalization * (GLOBAL_numberOfEvents*(2./(Double_t)BINS));
	if (x[0]<=1. && x[0]>=-1. && fullDistribution > 0) {
		return fullDistribution;
	}
	else {return 1e-30;}
}


Double_t unBiasedFullFitFunction(Double_t *x, Double_t *par) {
	Double_t dx = 2./(Double_t)BINS;
	Double_t collinsSoper     = 1.;
	Int_t binLocationIndex    = 0;
	Double_t efficiency       = 1.;
	Double_t normalization    = 1.;
	Double_t fullDistribution = 1.;
	if (x[0] <= 1. && x[0] >= -1.) {
		if (x[0]!=1.) { binLocationIndex = (Int_t)((x[0]+1.)/dx); }
		else          { binLocationIndex = BINS-1; }
		efficiency	 = GLOBAL_eff[binLocationIndex];
		collinsSoper	 = 1.+0.5*par[0] + par[1]*x[0] + (1.-1.5*par[0])*x[0]*x[0];
		normalization    = GLOBAL_efficiencySum * ( 1.+0.5*par[0] + par[1]*GLOBAL_cos1thetaAverage + (1.-1.5*par[0])*GLOBAL_cos2thetaAverage );
		fullDistribution = ( (efficiency * collinsSoper) / normalization ) * (GLOBAL_numberOfEvents*(2./(Double_t)BINS));
		if (fullDistribution > 0) { return fullDistribution; }
		else                      { return 1e-30; }
	}
	else {return 1e-30;}
}



int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;

	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	gStyle->SetOptStat(11);

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();

	// set the prefix
	string dirName = "/KKvsZE6vsSMstudy/";
	int iModel = 1;
	string sModel = "KK";
	cout << "KK(1), SM(2) or E6(3) ?" << endl;
	cout << "..." << endl;
	cin >> iModel;
	switch(iModel) {
		case 1: sModel = "KK"; break;
		case 2: sModel = "SM"; break;
		case 3: sModel = "E6"; break;
		default: cout << "wrong option, set to default: KK(1)" << endl;
			 sModel = "KK"; break;
			 break;
	}
	string outFileName = dirName + "Chi2/" + sModel + "_chi2collinsSoper";


	int sampleSizeSelector = 1;
	cout << "select sample size: 1M(1), L=500fb^-1(2), L=100fb^-1(3)..." << endl;
	cin >> sampleSizeSelector;
	if (sampleSizeSelector!=1 && sampleSizeSelector!=2 && sampleSizeSelector!=3) {
		cout << "invalid option, set to 1 (default)" << endl;
		sampleSizeSelector = 1;
	}
	string sSampleSize = "";
	switch (sampleSizeSelector) {
		case 1: sSampleSize = "_LrefinvFb"; break;
		case 2: sSampleSize = "_L500invFb"; break;
		case 3: sSampleSize = "_L100invFb"; break;
		default: break;
	}
	outFileName += sSampleSize;


	// declare the relevant classes
	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)
	dileptonAnalyzer anlz;
	Particle particle;

	// set cuts
	double invMassCut=1000.;
	double pTransverseCut=10.;
	double EtaCut=2.5;

	double no_invMassCut=0.;
	double no_pTransverseCut=0.;
	double no_EtaCut=1e10;


	// choose reference frame
	int referenceFrameSelector = 0;
	string cosThetaTitle;
	cout << "choose reference frame to calculate cos(theta)" << endl;
	cout << "(1) Helicity frame or (2) Collins-Soper frame" << endl;
	cout << "..." << endl;
	cin >> referenceFrameSelector;
	if (referenceFrameSelector != 1 && referenceFrameSelector != 2) {
		cout << "invalid option, setting to default = 1" << endl;
		referenceFrameSelector = 1;
	}
	if (referenceFrameSelector == 1) {cosThetaTitle = "Helicity";}
	else if (referenceFrameSelector == 2) {cosThetaTitle = "Collins-Soper";}


	int sampleSelector = 1;
	string withORwithoutCuts;
	cout << "select sample type: withCuts(1), noCuts(2)..." << endl;
	cin >> sampleSelector;
	if (sampleSelector!=1 && sampleSelector!=2) {
		cout << "wrong option: sampleSelector=" << sampleSelector << " setting to default (1)" << endl;
		sampleSelector = 1;
	}
	else {
		switch(sampleSelector) {
			case 1: withORwithoutCuts = "_withCuts"; break;
			case 2: withORwithoutCuts = "_noCuts"; break;
			default: break;
		}
	}
	//--- select static method of calculating costheta
	int quarkDirectionOrRapidity = 2; // 1=quark direction, 2=rapidity sign



	// declaration of histograms
	//---------------------------------------------------------------------------------------------------------
	Double_t cosThetamin   = -1.;
	Double_t cosThetamax   = 1.;

	// titles for the cos(theta) plots
	string title  = "";
	string xtitle = "cos#theta ' (" + cosThetaTitle + " frame)";
	string ytitle = "#left(#frac{dN}{dcos#theta '}#right)";

	TH1 *cosTheta_withCuts = new TH1D("", "", BINS, cosThetamin, cosThetamax);
	TH1 *cosTheta_noCuts = new TH1D("", "", BINS, cosThetamin, cosThetamax);
	
	cosTheta_withCuts->SetLineStyle(1);
	cosTheta_withCuts->SetMarkerStyle(20);
	cosTheta_withCuts->SetXTitle(xtitle.c_str());
	cosTheta_withCuts->SetYTitle("Events");

	cosTheta_noCuts->SetLineStyle(1);
	cosTheta_noCuts->SetMarkerStyle(20);
	cosTheta_noCuts->SetXTitle(xtitle.c_str());
	cosTheta_noCuts->SetYTitle("Events");
	//----------------------------------------------------------------------------------------------------------


	// declare kinematics histograms and root items
	string chi2collinssoperRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + withORwithoutCuts + ".root");
	string chi2collinssoperEpsFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + withORwithoutCuts + ".eps");


	TFile *chi2collinssoperHistFile = new TFile(chi2collinssoperRootFileName.c_str(),"recreate");
	TCanvas *canv = new TCanvas("canv","Kaluza-Klein chi2 collins soper",0,0,1400,800);


	string filePath;
	Int_t Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	Int_t Nleptons, Naleptons;
	Double_t cosThetaBoost, cosThetaCollinsSoper;
	Double_t invariantMass, cosTh=0;

	// to check the asymmetry in a narrow range arrownd the peak
	Double_t mZ_known = physPrm.mStar;
	Double_t mZ_lowLimit = physPrm.mStar-2000.;
	Double_t mZ_highLimit = physPrm.mStar+1000.;
	cout << "\n" << "mZ_known=" << mZ_known << "  mZ_lowLimit=" << mZ_lowLimit << "  mZ_highLimit=" << mZ_highLimit << "\n" << endl;
	


	// read the trees and put all good lepton pairs into 2 vectors
	if (sampleSizeSelector==1) { // sample size = 1M
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+sModel+"_extractedParticles_2trees_1Msample"+".root");
	}
	else if (sampleSizeSelector==2) { // L=500fb^-1
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+sModel+"_extractedParticles_2trees_500inversefemtobarnSample.root");
	}
	else if (sampleSizeSelector==3) { // L=100fb^-1
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100inversefemtobarn/"+sModel+"_extractedParticles_2trees_100inversefemtobarnSample.root");
	}
		
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);
	Nevents = anlz.loopOverEvents(filePath,"testTree", 0,0);
	cout << "Nevents = " << Nevents << endl;
	// get all the good pairs (after cuts)
	Nleptons = anlz.getAllLeptons(allgoodLeptons);
	Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
	cout << "Nleptons = " << Nleptons << endl;
	if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
	cout << "\n" << endl;
	
	// run over the good pairs (passed cuts) and fill the histograms
	for (int k=0 ; k<Nleptons ; k++) {
		invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
		cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
		if (quarkDirectionOrRapidity==2) {
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
		}
		else if (quarkDirectionOrRapidity==1) {
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k], true);
		}
		else {cout << "wrong option for quarkDirectionOrRapidity: " << quarkDirectionOrRapidity << " exitting now" << endl; exit(1);}
		if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
		else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
		// fill the histos
		if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			cosTheta_withCuts->Fill( (Double_t)cosTh );
		}
	}
	// clear the pairs vectors
	anlz.clearAllLeptons();
	if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
	if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }	
		
	//-------------------------------------------------------------------------------------------------------------------
	// no cuts
	anlz.setCuts(no_invMassCut, no_pTransverseCut, no_EtaCut);
	Nevents = anlz.loopOverEvents(filePath,"testTree_allPairs", 0,0);
	cout << "Nevents_allPairs = " << Nevents << endl;
	// get all the good pairs (no cuts)
	Nleptons = anlz.getAllLeptons(allgoodLeptons);
	Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
	cout << "Nleptons_allPairs = " << Nleptons << endl;
	if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
	cout << "\n" << endl;

	// run over the good pairs (passed cuts) and fill the histograms
	for (int k=0 ; k<Nleptons ; k++) {
		invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
		cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
		if (quarkDirectionOrRapidity==2) {
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
		}
		else if (quarkDirectionOrRapidity==1) {
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k], true);
		}
		if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
		else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
		// fill the histos
		if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			cosTheta_noCuts->Fill( (Double_t)cosTh );
		}
	}
	// clear the pairs vectors
	anlz.clearAllLeptons();
	if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
	if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }


	cosTheta_withCuts->Print();
	cosTheta_noCuts->Print();
	
	
	if      (sampleSelector==1) { GLOBAL_numberOfEvents = cosTheta_withCuts->GetEntries(); }
	else if (sampleSelector==2) { GLOBAL_numberOfEvents = cosTheta_noCuts->GetEntries(); }


	cout << "\n" << endl;



	// constants
	GLOBAL_efficiencySum    = 0.; // initialize
	GLOBAL_cos2thetaAverage = 0.; // initialize
	GLOBAL_cos1thetaAverage = 0.; // initialize
	Double_t cosTheta, ratio, withCuts=0., noCuts=0.;
	Double_t dCost = 2./(Double_t)BINS;
	for (Int_t n=0 ; n<BINS ; n++) {
		cosTheta = -1. + (1.+2*n)*0.5*dCost;
		withCuts = cosTheta_withCuts->GetBinContent(n+1);
		noCuts   = cosTheta_noCuts->GetBinContent(n+1);
		if (noCuts!=0) {
			ratio = withCuts/noCuts;
			GLOBAL_efficiencySum += ratio;
		}
		else { ratio = 1e-10; }
		GLOBAL_eff[n] = ratio;
		GLOBAL_cos1thetaAverage += cosTheta * ratio;
		GLOBAL_cos2thetaAverage += cosTheta*cosTheta * ratio;
	}
	GLOBAL_cos1thetaAverage /= GLOBAL_efficiencySum;
	GLOBAL_cos2thetaAverage /= GLOBAL_efficiencySum;
	GLOBAL_efficiencySum *= dCost;
	cout	<< "GLOBAL_cos1thetaAverage=" << GLOBAL_cos1thetaAverage << "\t"
		<< "GLOBAL_cos2thetaAverage=" << GLOBAL_cos2thetaAverage << "\t"
		<< "GLOBAL_efficiencySum=" << GLOBAL_efficiencySum << endl;



	cout << "\n" << endl;


	//---------- fit -----------
	Double_t chi2dof = 0.;
	Double_t A0  = 0., A4  = 0.;
        Double_t dA0 = 0., dA4 = 0.;


	TF1 *func = NULL;
	if      (sampleSelector==1) { func = new TF1("csDistFit",unBiasedFullFitFunction,-1.,1.,2); }
	else if (sampleSelector==2) { func = new TF1("csDistFit",physicalFitFunction,-1.,1.,2); }


	// initialize parameters
	func->SetParameter(0,0.); // parameter 0
	func->SetParLimits(0,-10.,10.);
        func->SetParameter(1,0.3); // parameter 1
	func->SetParLimits(1,-10.,10.);
	func->SetParNames("A0","A4");


        // make the fit
	if      (sampleSelector==1) { cosTheta_withCuts->Fit("csDistFit"); }
	else if (sampleSelector==2) { cosTheta_noCuts->Fit("csDistFit"); }


	// get the results
	TF1 *fitResult = NULL;
	if (sampleSelector==1) {
		fitResult = cosTheta_withCuts->GetFunction("csDistFit");
		cosTheta_withCuts->GetFunction("csDistFit")->SetLineWidth(1);
		cosTheta_withCuts->GetFunction("csDistFit")->SetLineColor(kBlack);
	}
	else if (sampleSelector==2) {
		fitResult = cosTheta_noCuts->GetFunction("csDistFit");
		cosTheta_noCuts->GetFunction("csDistFit")->SetLineWidth(1);
		cosTheta_noCuts->GetFunction("csDistFit")->SetLineColor(kBlack);
	}
	A0 = fitResult->GetParameter(0);
	dA0 = fitResult->GetParError(0);
	A4 = fitResult->GetParameter(1);
	dA4 = fitResult->GetParError(1);
        chi2dof = fitResult->GetChisquare()/fitResult->GetNDF();

	Double_t normalization = 8./3.;

	cout    << "Afb=" << (1./normalization) * A4 << "+/-" << (1./normalization) * dA4 << "\t" << endl;
	cout    << "A0=" << A0 << "+/-" << dA0 << "\t" << endl;
	cout	<< "chi2dof=" << chi2dof << endl;



	// ------ legends -----------------------------------------------------------------------------------------------------
	TLegend *leg = new TLegend(0.35,0.7,0.65,0.9);
	string label1 = "Generated data #left(" + sModel + "#right)";
	string label2 = "#chi^{2} fit #left(" + sModel + "#right)";
	if (iModel ==3) {
		label1 = "Generated data #left(Z'#right)";
		label2 = "#chi^{2} fit #left(Z'#right)";
	}
	if (sampleSelector==1) {
		leg->AddEntry(cosTheta_withCuts,label1.c_str(),"P");
		leg->AddEntry(cosTheta_withCuts->GetFunction("csDistFit"),label2.c_str(),"L");
	}
	else if (sampleSelector==2) {
		leg->AddEntry(cosTheta_noCuts,label1.c_str(),"P");
                leg->AddEntry(cosTheta_noCuts->GetFunction("csDistFit"),label2.c_str(),"L");
	}


	// frames
	Double_t maxHeight = 0;
	if      (sampleSelector==1) { maxHeight = cosTheta_withCuts->GetMaximum() + sqrt(cosTheta_withCuts->GetMaximum()); }
	else if (sampleSelector==2) { maxHeight = cosTheta_noCuts->GetMaximum() + sqrt(cosTheta_noCuts->GetMaximum()); }
	TH2 *frameHisto = new TH2D("", "", 1,cosThetamin,cosThetamax, 1,0.,1.05*maxHeight);
	frameHisto->SetXTitle(xtitle.c_str());
	frameHisto->SetYTitle("Events");


	cout << "\n---finalizing---\n" << endl;

	// draw the histos
	frameHisto->Draw();
	if      (sampleSelector==1) { cosTheta_withCuts->Draw("E1,sames"); }
	else if (sampleSelector==2) { cosTheta_noCuts->Draw("E1,sames"); }
	fitResult->Draw("sames");
	leg->Draw("sames");

	
	// finalize the graphics
	canv->Update();
	canv->Print(chi2collinssoperRootFileName.c_str(),"root");
	canv->Update();
	canv->Print(chi2collinssoperEpsFileName.c_str());
	chi2collinssoperHistFile->Write();

	// wait so the plots can hold on
	canv->WaitPrimitive();

	// Done.
	return 0;
}

