/*********************************************/
/* INTERFACE CLASSES TO Pythia8              */
/* feed pythia with a completely new process */
/*********************************************/
#include "pythiaSemiExternalPlugin.h"

using namespace std;
using namespace Pythia8; 

namespace Moses1 {

void PYTHIA8gmZ2mumubar::initProc() {
	// Process name.
	nameSave                  = "f fbar -> F Fbar (s-channel gamma*/Z0)";
	if (idNew == 4) nameSave  = "f fbar -> c cbar (s-channel gamma*/Z0)";
	if (idNew == 5) nameSave  = "f fbar -> b bbar (s-channel gamma*/Z0)";
	if (idNew == 6) nameSave  = "f fbar -> t tbar (s-channel gamma*/Z0)";
	if (idNew == 7) nameSave  = "f fbar -> b' b'bar (s-channel gamma*/Z0)";
	if (idNew == 8) nameSave  = "f fbar -> t' t'bar (s-channel gamma*/Z0)";
	if (idNew == 15) nameSave = "f fbar -> tau+ tau- (s-channel gamma*/Z0)";
	if (idNew == 17) nameSave = "f fbar -> tau'+ tau'- (s-channel gamma*/Z0)";
	if (idNew == 18) nameSave = "f fbar -> nu'_tau nu'bar_tau (s-channel gamma*/Z0)";
	
	// Allow to pick only gamma* or Z0 part of full gamma*/Z0 expression.
	gmZmode      = settingsPtr->mode("WeakZ0:gmZmode");

	// Store Z0 mass and width for propagator. 
	mRes         = particleDataPtr->m0(23);
	GammaRes     = particleDataPtr->mWidth(23);
	m2Res        = mRes*mRes;
	GamMRat      = GammaRes / mRes;
	thetaWRat    = 1. / (16. * coupSMPtr->sin2thetaW() * coupSMPtr->cos2thetaW());

	// Store couplings of F.
	ef           = coupSMPtr->ef(idNew);
	vf           = coupSMPtr->vf(idNew);
	af           = coupSMPtr->af(idNew);

	// Secondary open width fraction, relevant for top (or heavier).
	openFracPair = particleDataPtr->resOpenFrac(idNew, -idNew);
}


//--------------------------------------------------------------------------


// Evaluate d(sigmaHat)/d(tHat), part independent of incoming flavour. 
void PYTHIA8gmZ2mumubar::sigmaKin() { 

	// Check that above threshold.
	isPhysical     = true;
	if (mH < m3 + m4 + MASSMARGIN) {
		isPhysical   = false;
		return;
	}

	// Define average F, Fbar mass so same beta. Phase space.
	double s34Avg  = 0.5 * (s3 + s4) - 0.25 * pow2(s3 - s4) / sH; 
	mr             = s34Avg / sH;
	betaf          = sqrtpos(1. - 4. * mr);

	// Final-state colour factor.
	double colF    = (idNew < 9) ? 3. * (1. + alpS / M_PI) : 1.;
 
	// Reconstruct decay angle so can reuse 2 -> 1 cross section.
	cosThe         = (tH - uH) / (betaf * sH); 

	// Calculate prefactors for gamma/interference/Z0 cross section terms.
	gamProp = colF * M_PI * pow2(alpEM) / sH2; 
	intProp = gamProp * 2. * thetaWRat * sH * (sH - m2Res)
			/ ( pow2(sH - m2Res) + pow2(sH * GamMRat) );
	resProp = gamProp * pow2(thetaWRat * sH) 
			/ ( pow2(sH - m2Res) + pow2(sH * GamMRat) );

	// Optionally only keep gamma* or Z0 term.
	if (gmZmode == 1) {intProp = 0.; resProp = 0.;}
	if (gmZmode == 2) {gamProp = 0.; intProp = 0.;}
}


//--------------------------------------------------------------------------


// Evaluate d(sigmaHat)/d(tHat), including incoming flavour dependence. 
double PYTHIA8gmZ2mumubar::sigmaHat() { 

	// Fail if below threshold.
	if (!isPhysical) return 0.;

	// Couplings for in-flavours.
	int idAbs       = abs(id1);
	double ei       = coupSMPtr->ef(idAbs);
	double vi       = coupSMPtr->vf(idAbs);
	double ai       = coupSMPtr->af(idAbs);

	// Coefficients of angular expression.
	double coefTran = ei*ei * gamProp * ef*ef + ei * vi * intProp * ef * vf
					+ (vi*vi + ai*ai) * resProp * (vf*vf + pow2(betaf) * af*af);
	double coefLong = 4. * mr * ( ei*ei * gamProp * ef*ef 
					+ ei * vi * intProp * ef * vf + (vi*vi + ai*ai) * resProp * vf*vf );
	double coefAsym = betaf * ( ei * ai * intProp * ef * af 
					+ 4. * vi * ai * resProp * vf * af );
	
	// Combine gamma, interference and Z0 parts.
	double sigma    = coefTran * (1. + pow2(cosThe)) 
					+ coefLong * (1. - pow2(cosThe)) + 2. * coefAsym * cosThe; 

	// Top: corrections for closed decay channels.
	sigma *= openFracPair;

	// Initial-state colour factor. Answer.
	if (idAbs < 9) sigma /= 3.;
	return sigma;
}


//--------------------------------------------------------------------------


// Select identity, colour and anticolour.
void PYTHIA8gmZ2mumubar::setIdColAcol() {
	
	// Set outgoing flavours.
	id3 = (id1 > 0) ? idNew : -idNew;
	setId( id1, id2, id3, -id3);

	// Colour flow topologies. Swap when antiquarks.
	if (abs(id1) < 9 && idNew < 9) setColAcol( 1, 0, 0, 1, 2, 0, 0, 2);
	else if (abs(id1) < 9)         setColAcol( 1, 0, 0, 1, 0, 0, 0, 0);
	else if (idNew < 9)            setColAcol( 0, 0, 0, 0, 1, 0, 0, 1);
	else                           setColAcol( 0, 0, 0, 0, 0, 0, 0, 0);
	if (id1 < 0) swapColAcol();
}


//--------------------------------------------------------------------------


// Evaluate weight for decay angles of W in top decay.
double PYTHIA8gmZ2mumubar::weightDecay( Event& process, int iResBeg, int iResEnd) {

	// For top decay hand over to standard routine, else done.
	if (idNew == 6 && process[process[iResBeg].mother1()].idAbs() == 6) 
		return weightTopDecay( process, iResBeg, iResEnd);
	else return 1.; 
}


}
